// Copyright (c) rubicon IT GmbH, www.rubicon.eu
//
// See the NOTICE file distributed with this work for additional information
// regarding copyright ownership.  rubicon licenses this file to you under 
// the Apache License, Version 2.0 (the "License"); you may not use this 
// file except in compliance with the License.  You may obtain a copy of the 
// License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software 
// distributed under the License is distributed on an "AS IS" BASIS, WITHOUT 
// WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the 
// License for the specific language governing permissions and limitations
// under the License.
// 
using System;
using System.Linq.Expressions;
using NUnit.Framework;
using Remotion.Linq.Development.UnitTesting;
using Remotion.Linq.UnitTests.Clauses.Expressions.TestDomain;
using Rhino.Mocks;

namespace Remotion.Linq.UnitTests.Clauses.Expressions
{
  [TestFixture]
  public class ExtensionExpressionIntegrationTest
  {
    [Test]
    public void ExpressionOverridingAcceptMethod ()
    {
      var visitorMock = MockRepository.GenerateMock<SpecificVisitor> ();
      
      var expression = new TestableExtensionExpressionWithSpecificVisitor (typeof (int));
      ExtensionExpressionTestHelper.CallAccept (expression, visitorMock);

      visitorMock.AssertWasCalled (mock => mock.VisitTestableExtensionExpression (expression));
    }

    [Test]
    public void CanReduce_True ()
    {
      var expression = new ReducibleExtensionExpression (typeof (int));
      Assert.That (expression.CanReduce, Is.True);
    }

    [Test]
    public void Reduce ()
    {
      var expression = new ReducibleExtensionExpression (typeof (int));

      var result = expression.Reduce ();

      var expectedExpression = Expression.Constant (0);
      ExpressionTreeComparer.CheckAreEqualTrees (expectedExpression, result);
    }
  }
}